@@ -51,14 +51,14 @@ class InfiniteValue(Number):
         self.floatvalue = float("-inf" if self.negative else "inf")
 
     def __lt__(self, other):
-        """ -infinity is always less """
+        """-infinity is always less"""
         if isinstance(other, float):
             return self.floatvalue < other
         else:
             return self.negative and not self == other
 
     def __gt__(self, other):
-        """ +infinity is always more """
+        """+infinity is always more"""
         if isinstance(other, float):
             return self.floatvalue > other
         else:
@@ -74,7 +74,7 @@ class InfiniteValue(Number):
         return not self == other
 
     def __eq__(self, other):
-        """ for consistency, infinity is equal to itself """
+        """for consistency, infinity is equal to itself"""
         if isinstance(other, float):
             return self.floatvalue == other
         elif isinstance(other, InfiniteValue):
@@ -137,17 +137,20 @@ class InfiniteValue(Number):
         return str(self.floatvalue)
 
     def __repr__(self):
-        """ pretend to be float infinity """
+        """pretend to be float infinity"""
         return repr(self.floatvalue)
 
     def __hash__(self):
-        """ pretend to be float infinity """
+        """pretend to be float infinity"""
         return hash(self.floatvalue)
 
     def __index__(self) -> int:
         return maxsize
 
 
+Inf = InfiniteValue()
+
+
 class _LinkedList(Iterable):
     """
     A custom definition of a single, feature-poor, linked-list.
@@ -351,7 +354,7 @@ class _LinkedList(Iterable):
         self.node_at(index).value = value
 
     def find_node(self, value):
-        """ Returns the node that contains the given value """
+        """Returns the node that contains the given value"""
         cur = self.first
         while cur:
             if cur.value == value:
@@ -441,3 +444,196 @@ class _LinkedList(Iterable):
 
     def __str__(self):
         return f"LinkedList{str(list(iter(self)))}"
+
+
+class _Sentinel(object):
+    pass
+
+
+class _UnhashableFriendlyDict(dict):
+    def __init__(self, *args, **kwargs):
+        self._unhashable = []
+        super(_UnhashableFriendlyDict, self).__init__()
+        self.update(*args, **kwargs)
+
+    def __setitem__(self, key, value):
+        try:
+            super(_UnhashableFriendlyDict, self).__setitem__(key, value)
+        except TypeError:
+            for i in range(len(self._unhashable)):
+                if self._unhashable[i][0] == key:
+                    del self[self._unhashable[i][2]]
+                    self._unhashable.pop(i)
+                    break
+            sentinel = _Sentinel()
+            self._unhashable.append((key, value, sentinel))
+            self[sentinel] = sentinel
+
+    def __getitem__(self, item):
+        try:
+            return super(_UnhashableFriendlyDict, self).__getitem__(item)
+        except TypeError:
+            for k, v, _ in self._unhashable:
+                if k == item:
+                    return v
+            raise KeyError(item)
+
+    def __delitem__(self, key):
+        try:
+            super(_UnhashableFriendlyDict, self).__delitem__(key)
+        except TypeError:
+            for i in range(len(self._unhashable)):
+                if self._unhashable[i][0] == key:
+                    del self[self._unhashable[i][2]]
+                    self._unhashable.pop(i)
+                    break
+
+    def __contains__(self, item):
+        try:
+            return super(_UnhashableFriendlyDict, self).__contains__(item)
+        except TypeError:
+            return item in (i[0] for i in self._unhashable)
+
+    def __eq__(self, other):
+        if self._unhashable:
+            return (
+                isinstance(other, _UnhashableFriendlyDict)
+                and super(_UnhashableFriendlyDict, self).__eq__(other)
+                and self._unhashable == other.unhashable
+            )
+        return super(_UnhashableFriendlyDict, self).__eq__(other)
+
+    def __iter__(self):
+        for key in super(_UnhashableFriendlyDict, self).__iter__():
+            if isinstance(key, _Sentinel):
+                for k, _, s in self._unhashable:
+                    if s is key:
+                        yield k
+                        break
+                else:
+                    yield key
+            else:
+                yield key
+
+    def __len__(self):
+        return super(_UnhashableFriendlyDict, self).__len__() + len(self._unhashable)
+
+    def __reversed__(self):
+        for key in super(_UnhashableFriendlyDict, self).__reversed__():
+            if isinstance(key, _Sentinel):
+                for k, _, s in self._unhashable:
+                    if s is key:
+                        yield k
+                        break
+                else:
+                    yield key
+            else:
+                yield key
+
+    def __repr__(self):
+        return f"""{{{
+            ', '.join([f'{repr(key)}: {repr(value)}' for key, value in self.items()])
+        }}}"""
+
+    def __str__(self):
+        return repr(self)
+
+    def clear(self):
+        self._unhashable.clear()
+        super(_UnhashableFriendlyDict, self).clear()
+
+    def copy(self):
+        c = _UnhashableFriendlyDict(super(_UnhashableFriendlyDict, self).copy())
+        c.unhashable = self._unhashable[:]
+
+    def get(self, key, default=None):
+        try:
+            return self[key]
+        except KeyError:
+            return default
+
+    def items(self):
+        for key, value in super(_UnhashableFriendlyDict, self).items():
+            if isinstance(key, _Sentinel):
+                for k, v, s in self._unhashable:
+                    if s is key:
+                        yield (k, v)
+                        break
+                else:
+                    yield (key, value)
+            else:
+                yield (key, value)
+
+    def keys(self):
+        for key in super(_UnhashableFriendlyDict, self).keys():
+            if isinstance(key, _Sentinel):
+                for k, _, s in self._unhashable:
+                    if s is key:
+                        yield k
+                        break
+                else:
+                    yield key
+            else:
+                yield key
+
+    def pop(self, d):
+        v = self[d]
+        del self[d]
+        return v
+
+    def popitem(self):
+        k, v = super(_UnhashableFriendlyDict, self).popitem()
+        if isinstance(k, _Sentinel):
+            for i in range(len(self._unhashable)):
+                if self._unhashable[i][2] is k:
+                    nk, nv = self._unhashable[i][0], self._unhashable[i][1]
+                    self._unhashable.pop(i)
+                    del self[self._unhashable[i][2]]
+                    return nk, nv
+            else:
+                return k, v
+        else:
+            return k, v
+
+    def setdefault(self, key, default=None):
+        try:
+            return super(_UnhashableFriendlyDict, self).setdefault(key, default)
+        except TypeError:
+            if key not in self:
+                self[key] = default
+                return default
+            return self[key]
+
+    def update(self, *args, **f):
+        if len(args) > 1:
+            raise TypeError(f"update expected at most 1 arguments, got {len(args)}")
+        if len(args) == 1:
+            e = args[0]
+            if hasattr(e, "keys") and callable(e.keys):
+                for k in e:
+                    self[k] = e[k]
+            else:
+                for k, v in e:
+                    self[k] = v
+            if isinstance(e, _UnhashableFriendlyDict):
+                self._unhashable.extend(e._unhashable)
+        for k in f:
+            self[k] = f[k]
+
+    def values(self):
+        for value in super(_UnhashableFriendlyDict, self).values():
+            if isinstance(value, _Sentinel):
+                for _, v, s in self._unhashable:
+                    if s is value:
+                        yield v
+                        break
+                else:
+                    yield value
+            else:
+                yield value
+
+    @classmethod
+    def fromkeys(cls, iterable, value=None):
+        d = _UnhashableFriendlyDict()
+        for item in iterable:
+            d[item] = value
