@@ -27,6 +27,8 @@ from anchorpoint.utils._helper import (
     _LinkedList,
     InfiniteValue,
     _is_iterable_non_string,
+    Inf,
+    _UnhashableFriendlyDict,
 )
 
 
@@ -67,8 +69,10 @@ class Range:
 
           >>> e = Range(3, 5)
           >>> f = Range(3, 5, include_start=False, include_end=True)
-          >>> print(e)  # [3, 5)
-          >>> print(f)  # (3, 5]
+          >>> e
+          Range[3, 5)
+          >>> f
+          Range(3, 5]
 
         4. From the keyword arguments `start` and/or `end`. `start` and `end`
           may be anything so long as the condition `start <= end` is True and
@@ -330,9 +334,7 @@ class Range:
             raise TypeError("Cannot overlap a Range with a non-Range")
         # do the ranges overlap?
         rng_a, rng_b = (self, rng) if self < rng else (rng, self)
-        if rng_a.isdisjoint(rng_b) and not (
-            rng_a.end == rng_b.start and rng_a.include_end != rng_b.include_start
-        ):
+        if rng_a.isdisjoint(rng_b):
             return None
         # compute parameters for new intersecting range
         # new_start = rng_b.start
@@ -486,7 +488,7 @@ class Range:
         )
 
     def copy(self):
-        """ Returns a copy of this object, identical to calling `Range(self)` """
+        """Returns a copy of this object, identical to calling `Range(self)`"""
         return Range(self)
 
     def length(self):
@@ -527,13 +529,29 @@ class Range:
             f"Range of {self.start.__class__} to {self.end.__class__} has no defined length"
         )
 
+    def isinfinite(self):
+        """
+        Returns True if this Range has a negative bound of -Inf or a positive bound of +Inf
+        """
+        return self.start == -Inf or self.end == Inf
+
     def _above_start(self, item):
+        if isinstance(item, Range):
+            if self.include_start or self.include_start == item.include_start:
+                return item.start >= self.start
+            else:
+                return item.start > self.start
         if self.include_start:
             return item >= self.start
         else:
             return item > self.start
 
     def _below_end(self, item):
+        if isinstance(item, Range):
+            if self.include_end or self.include_end == item.include_end:
+                return item.end <= self.end
+            else:
+                return item.end < self.end
         if self.include_end:
             return item <= self.end
         else:
@@ -707,7 +725,7 @@ class Range:
 
     def __repr__(self):
         return (
-            f"Range{'[' if self.include_start else '('}{repr(self.start)}, "
+            f"{self.__class__.__name__}{'[' if self.include_start else '('}{repr(self.start)}, "
             f"{repr(self.end)}{']' if self.include_end else ')'}"
         )
 
@@ -817,7 +835,7 @@ class RangeSet(Iterable):
                     inserted_node.value = prev_union
                     temp_ranges.pop_before(inserted_node)
                     prev_union = (
-                        inserted_node.value.union(inserted_node.prev)
+                        inserted_node.value.union(inserted_node.prev.value)
                         if inserted_node.prev
                         else None
                     )
@@ -828,7 +846,7 @@ class RangeSet(Iterable):
                     inserted_node.value = next_union
                     temp_ranges.pop_after(inserted_node)
                     next_union = (
-                        inserted_node.value.union(inserted_node.next)
+                        inserted_node.value.union(inserted_node.next.value)
                         if inserted_node.next
                         else None
                     )
@@ -1076,6 +1094,15 @@ class RangeSet(Iterable):
         """
         return self.__class__(self)
 
+    def isinfinite(self):
+        """
+        Returns True if this RangeSet has a negative bound of -Inf or a positive bound of +Inf,
+        and False otherwise
+        """
+        return (
+            self._ranges.first.value.start == -Inf or self._ranges.last.value.end == Inf
+        )
+
     @staticmethod
     def _merge_ranges(ranges):
         """
@@ -1200,48 +1227,48 @@ class RangeSet(Iterable):
         return not self == other
 
     def __and__(self, other):
-        """ returns (self & other), identical to self.intersection(other) """
+        """returns (self & other), identical to self.intersection(other)"""
         return self.intersection(other)
 
     def __or__(self, other):
-        """ returns (self | other), identical to self.union(other) """
+        """returns (self | other), identical to self.union(other)"""
         return self.union(other)
 
     def __xor__(self, other):
-        """ returns (self ^ other), identical to
-        self.symmetric_difference(other) """
+        """returns (self ^ other), identical to
+        self.symmetric_difference(other)"""
         return self.symmetric_difference(other)
 
     def __sub__(self, other):
-        """ returns (self - other), identical to self.difference(other) """
+        """returns (self - other), identical to self.difference(other)"""
         return self.difference(other)
 
     def __add__(self, other):
-        """ Returns (self + other), identical to self.union(other) """
+        """Returns (self + other), identical to self.union(other)"""
         return self.union(other)
 
     def __iand__(self, other):
-        """ Executes (self &= other), identical to self.intersection_update(other) """
+        """Executes (self &= other), identical to self.intersection_update(other)"""
         self.intersection_update(other)
         return self
 
     def __ior__(self, other):
-        """ Executes (self |= other), identical to self.update(other) """
+        """Executes (self |= other), identical to self.update(other)"""
         self.update(other)
         return self
 
     def __ixor__(self, other):
-        """ Executes (self ^= other), identical to self.symmetric_difference_update(other) """
+        """Executes (self ^= other), identical to self.symmetric_difference_update(other)"""
         self.symmetric_difference_update(other)
         return self
 
     def __iadd__(self, other):
-        """ Executes (self += other), identical to self.update(other) """
+        """Executes (self += other), identical to self.update(other)"""
         self.update(other)
         return self
 
     def __isub__(self, other):
-        """ Executes (self -= other), identical to self.difference_update(other) """
+        """Executes (self -= other), identical to self.difference_update(other)"""
         self.difference_update(other)
         return self
 
@@ -1286,9 +1313,12 @@ class RangeDict:
     ...     Range('h', 'p'): "Second third of the lowercase alphabet",
     ...     Range('p', '{'): "Final third of the lowercase alphabet",
     ... })
-    >>> print(c['brian'])  # First third of the lowercase alphabet
-    >>> print(c['king arthur'])  # Second third of the lowercase alphabet
-    >>> print(c['python'])  # Final third of the lowercase alphabet
+    >>> c['brian']
+    'First third of the lowercase alphabet'
+    >>> c['king arthur']
+    'Second third of the lowercase alphabet'
+    >>> c['python']
+    'Final third of the lowercase alphabet'
 
     >>> # From an iterable of 2-tuples, like a regular dict
     >>> d = RangeDict([
@@ -1360,6 +1390,8 @@ class RangeDict:
         iterable of 2-tuples (range, value).
         """
         # Internally, RangeDict has two data structures
+        #    _values is a dict {value: [rangeset, ...], ..., '_sentinel': [(value: [rangeset, ...]), ...]}
+        #          The sentinel allows the RangeDict to accommodate unhashable types.
         #    _values is a dict {value: [rangeset1, rangeset2, ...]}
         #    _ranges is a list-of-lists, [[(intrangeset1, value1), (intrangeset2, value2), ...],
         #                                 [(strrangeset1, value1), (strrangeset2, value2), ...],
@@ -1370,18 +1402,17 @@ class RangeDict:
         #  _ranges for the value we want to point to.
         # Meanwhile, _ranges is a list-of-lists instead of just a list, so that we can accommodate ranges of
         #  different types (e.g. a RangeSet of ints and a RangeSet of strings) pointing to the same values.
+        self._values = _UnhashableFriendlyDict()
         if iterable is RangeDict._sentinel:
-            self._values = {}
             self._rangesets = _LinkedList()
         elif isinstance(iterable, RangeDict):
-            self._values = {
-                val: rngsets[:] for val, rngsets in iterable._values.items()
-            }
+            self._values.update(
+                {val: rngsets[:] for val, rngsets in iterable._values.items()}
+            )
             self._rangesets = _LinkedList(
                 [rngset.copy() for rngset in iterable._rangesets]
             )
         elif isinstance(iterable, dict):
-            self._values = {}
             self._rangesets = _LinkedList()
             for rng, val in iterable.items():
                 if _is_iterable_non_string(rng):
@@ -1394,7 +1425,6 @@ class RangeDict:
                 assert _is_iterable_non_string(
                     iterable
                 )  # creative method of avoiding code reuse!
-                self._values = {}
                 self._rangesets = _LinkedList()
                 for rng, val in iterable:
                     # this should not produce an IndexError. It produces a TypeError instead.
@@ -1407,6 +1437,7 @@ class RangeDict:
                         self.add(rng, val)
             except (TypeError, ValueError, AssertionError):
                 raise ValueError("Expected a dict, RangeDict, or iterable of 2-tuples")
+        self._values[RangeDict._sentinel] = []
         self.popempty()
 
     def add(self, rng, value):
@@ -1878,7 +1909,7 @@ class RangeDict:
         return RangeDict(self)
 
     def _sort_ranges(self):
-        """ Helper method to gnomesort all _LinkedLists-of-RangeSets. """
+        """Helper method to gnomesort all _LinkedLists-of-RangeSets."""
         for linkedlist in self._rangesets:
             linkedlist.gnomesort()
 
